//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : CBkArray.h
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1998, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// Header file for the supporting bookmark array class
//
//=============================================================================

#if !defined(__CBKARRAY_H)
#define __CBKARRAY_H

#if !defined(assert)
#include <assert.h>
#endif

// Bookmark information.  Bookmarks are stored with the virtual memory
// buffer so that multiple owners have access to the same set of bookmarks.
// The bookmark data is manipulated at the editor level though to allow
// specialized implementations without deriving a new virtual memory class.

// All other values are user-defined types
#define BK_ALLTYPES     0       // Any type for delete
#define BK_NORMAL       1       // Normal bookmark

// Special "line" values.
// LN_PREVIOUS and LN_NEXT cause the find function to go to the nearest
// bookmark before or after the current editor line.  LN_FIRST and LN_LAST
// cause it to go to the first or last one of the given type.
#define LN_FIRST        -1
#define LN_PREVIOUS     -2
#define LN_NEXT         -3
#define LN_LAST         -4

// The bookmarks are stored in order by type and line number
struct Bookmark;

struct Bookmark
{
    int   nType;        // BK_NORMAL or a user-defined type
    long  lLine;        // Line number
    void *vpData;       // User-defined data (optional)

    // Change notification callback (optional)
    void (*vfpCallback)(Bookmark *);
};

// This is a simple helper class for the bookmark array.  It can
// be replaced with something more elaborate if necessary
class CBkArray
{
private:
    int nSize, nIdx;
    Bookmark **pData;

public:
    CBkArray();
    ~CBkArray();

    // Helper for indexing
	inline Bookmark * &operator[](int nSubscript);

    // Add/remove bookmarks
    Bookmark *Add(int nType, long lLine, void (*vfpCallback)(Bookmark *),
        void *vpData);
    void Remove(int nIndex);

    // Access functions
    inline int GetCount(void);
    inline const Bookmark **GetDataPtr();
};

// Inline function bodies
inline Bookmark * &CBkArray::operator[](int nSubscript)
{
    assert(pData && nSubscript > -1 && nSubscript < nIdx);

    return pData[nSubscript];
}

inline int CBkArray::GetCount(void)
    { return nIdx; }

inline const Bookmark **CBkArray::GetDataPtr()
    { return (const Bookmark **)pData; }

#endif      // __CBKARRAY_H
