//=============================================================================
// System  : CVMEditor - Virtual Memory Text Editor Library
// File    : EditorDialog.cpp
// Author  : Eric Woodruff  (Eric_Woodruff@CompuServe.com)
// Updated : Wed 07/08/98 01:00:00
// Note    : Copyright 1997-98, Eric Woodruff, All rights reserved
// Compiler: Visual C++
//
// This file contains the editor dialog support function
//
// Version    Date    Who   Comments
// ============================================================================
// 1.0.0    06/12/97  EFW   Ported the code to Visual C++
//=============================================================================

#include "stdafx.h"

#include <stdarg.h>
#include <stdio.h>

#include "CVMEditorDefs.h"
#include "CVMEResource.h"

#include "Resource.h"
#include "MFCEditor.h"
#include "MFCEditorDoc.h"
#include "MFCEditorView.h"

#include "LocalEditorOpts.h"
#include "ReplacePrompt.h"
#include "GotoJumpDlg.h"
#include "FindTextDlg.h"
#include "ReplaceTextDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//=============================================================================

//
// If you replace the default editor dialog function, be sure to check for
// all possible cases that your application will need!
//
// This one is the general editor dialog handler for any application.
//
int doEditDialog(int nDialog, ...)
{
    OPENFILENAME ofnParms;

    va_list arg;
    char msgText[256], *ln, szFilename[MAX_PATH], *pszFName;
    int  size;
    BOOL bRetVal;
    int  result = IDCANCEL;       // Default to cancel in case undefined

    HWND hWnd = ::GetFocus();

    va_start(arg, nDialog);

    switch(nDialog)
    {
        case edOutOfMemory:
            MessageBox(hWnd, "Not enough memory to complete operation",
                "Editor Error", MB_OK  | MB_ICONSTOP);
            break;

        case edReadError:       // Passes a char * to the name of the file
            sprintf(msgText, "Error reading file %s",
                (char *)va_arg(arg, char *));
            MessageBox(hWnd, msgText, "Editor Error", MB_OK | MB_ICONSTOP);
            break;

        case edWriteError:      // Passes a char * to the name of the file
            sprintf(msgText, "Error writing to %s",
                (char *)va_arg(arg, char *));
            MessageBox(hWnd, msgText, "Editor Error", MB_OK | MB_ICONSTOP);
            break;

        case edCreateError:     // Passes a char * to the name of the file
            sprintf(msgText, "Error creating file %s",
                (char *)va_arg(arg, char *));
            MessageBox(hWnd, msgText, "Editor Error", MB_OK | MB_ICONSTOP);
            break;

        case edSwapBadSeek:
            MessageBox(hWnd, "ERROR: Bad seek on swap file!", "Editor Error",
                MB_OK | MB_ICONSTOP);
            break;

        case edSwapFileFail:
            MessageBox(hWnd, "ERROR: Could not read/write swap file!",
                "Editor Error", MB_OK | MB_ICONSTOP);
            break;

        // Used by the diagnostic checks within the editor and virtual
        // memory manager.  Also used by TVMMemo::valid() if the
        // memo text is too long.  Passes a char * to the message string.
        case edGeneralMsg:
            MessageBox(hWnd, va_arg(arg, char *), "Editor Information",
                MB_OK | MB_ICONEXCLAMATION);
            break;

        case edOpCancelled:
            MessageBox(hWnd, "Operation cancelled by user",
                "Editor Information", MB_OK  | MB_ICONEXCLAMATION);
            break;

        case edLinesSplit:
            MessageBox(hWnd, "Long lines were split when loaded",
                "Editor Information", MB_OK  | MB_ICONEXCLAMATION);
            break;

        case edLineTooLong:
            MessageBox(hWnd, "Line would be too long", "Editor Information",
                MB_OK | MB_ICONEXCLAMATION);
            break;

        case edGotoLine:
            {
                // The input is *text* so that + or - can be used!
                // Absence of the + or - means a literal line number.
                // Passes a char * to the buffer and an int for the
                // maximum size of the string.
                ln = (char *)va_arg(arg, char *);
                size = (int)va_arg(arg, int);

                CGotoJumpDlg *gotoDlg = new CGotoJumpDlg;

                result = gotoDlg->DoModal();
                if(result == IDOK)
                {
                    strncpy(ln, gotoDlg->m_csLineNbr, size);
                    ln[size - 1] = '\x0';
                }
                delete gotoDlg;
            }
            break;

        case edChgLocalOpts:    // Passes addr of LocalEditorOptions structure
            {
                CLocalEditorOpts *optsDlg = new CLocalEditorOpts;

                LocalEditorOptions *opts = (LocalEditorOptions *)va_arg(
                    arg, LocalEditorOptions *);

                optsDlg->SetOptions(opts);

                if(optsDlg->DoModal() == IDOK)
                    optsDlg->GetOptions(opts);

                delete optsDlg;
            }
            break;

        case edFind:            // Passes addr of FindReplaceStruct structure
            {
                // As long as the options passed back are the same, this
                // could probably use the Find/Replace common dialog

                FindReplaceStruct *frData = (FindReplaceStruct *)va_arg(arg,
                    FindReplaceStruct *);

                CFindTextDlg *findDlg = new CFindTextDlg;
                findDlg->SetOptions(frData);

                result = findDlg->DoModal();
                if(result != IDCANCEL)
                    findDlg->GetOptions(frData);

                delete findDlg;
            }
            break;

        case edReplace:         // Passes addr of FindReplaceStruct structure
            // If the Replace Dialog returns IDYES, *ALL* occurances
            // will be replaced, not just the first one.  If IDOK is returned,
            // only the first occurance will be replaced.
            {
                // As long as the options passed back are the same, this
                // could probably use the Find/Replace common dialog

                FindReplaceStruct *frData = (FindReplaceStruct *)va_arg(arg,
                    FindReplaceStruct *);

                CReplaceTextDlg *repDlg = new CReplaceTextDlg;
                repDlg->SetOptions(frData);

                result = repDlg->DoModal();
                if(result != IDCANCEL)
                    repDlg->GetOptions(frData);

                delete repDlg;
            }
            break;

        case edReplacePrompt:
            {
                CMFCEditorView *pView;

#if !defined(USE_MDI)
                CFrameWnd* pFrame =
                    dynamic_cast<CFrameWnd*>(AfxGetApp()->m_pMainWnd);
#else
                CMDIFrameWnd* pFrame =
                    dynamic_cast<CMDIFrameWnd*>(AfxGetApp()->m_pMainWnd);
#endif
                if(!pFrame)
                    return IDNO;

#if defined(USE_MDI)
                CMDIChildWnd *pWnd =
                    dynamic_cast<CMDIChildWnd*>(pFrame->MDIGetActive());
                if(pWnd)
                    pView =
                        dynamic_cast<CMFCEditorView*>(pWnd->GetActiveView());
                else
                    pView = NULL;
#else
                pView = dynamic_cast<CMFCEditorView*>(pFrame->GetActiveView());
#endif

                if(!pView)
                    return IDNO;

                // Draw current view to show the highlighted text
                pView->GetDocument()->UpdateAllViews(NULL);

                CReplacePrompt *repDlg = new CReplacePrompt;

                // Avoid placing the dialog on the same line as the cursor
                CVMPoint *pnt = (CVMPoint *)va_arg(arg, CVMPoint *);
                repDlg->m_Point.x = 0;
                repDlg->m_Point.y = pnt->y * pView->editor->CharSize.cy;
                pView->ClientToScreen(&repDlg->m_Point);

                result = repDlg->DoModal();
                delete repDlg;
            }
            break;

        case edSearchFailed:
            MessageBox(hWnd, "Search string not found.", "Search Failed",
                MB_OK | MB_ICONEXCLAMATION);
            break;

        case edReadBlock:       // All pass a char * to a filename buffer
        case edSaveBlock:
        case edSaveAs:
            {
                memset(&ofnParms, 0, sizeof(OPENFILENAME));
                pszFName = (char *)va_arg(arg, char *);
                strcpy(szFilename, pszFName);

                ofnParms.lStructSize = sizeof(OPENFILENAME);
            	ofnParms.lpstrFilter =
                    "Text files(*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0";
                ofnParms.lpstrFile = szFilename;
        	    ofnParms.nMaxFile = sizeof(szFilename);
            	ofnParms.lpstrDefExt = "TXT";

                switch(nDialog)
                {
                    case edReadBlock:
                        ofnParms.lpstrTitle = "Read Block";
                        ofnParms.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY;
                        bRetVal = GetOpenFileName(&ofnParms);
                        break;

                    case edSaveBlock:
                        ofnParms.lpstrTitle = "Save Block";
                        ofnParms.Flags = OFN_PATHMUSTEXIST |
                            OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
                        bRetVal = GetSaveFileName(&ofnParms);
                        break;

                    case edSaveAs:
                        ofnParms.lpstrTitle = "Save File As";
                        ofnParms.Flags = OFN_PATHMUSTEXIST |
                            OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
                        bRetVal = GetSaveFileName(&ofnParms);
                        break;

                    default:
                        break;
                }

                if(bRetVal)
                {
                    strcpy(pszFName, szFilename);
                    result = IDOK;
                }
            }
            break;

        case edSaveUntitled:
            result = MessageBox(hWnd, "Save untitled file?", "Untitled File",
                MB_YESNOCANCEL | MB_ICONQUESTION);
            break;

        case edSaveModify:      // Passes char * to the name of the file
            sprintf(msgText, "%s has been modified.  Save?",
                (char *)va_arg(arg, char *));

            result = MessageBox(hWnd, msgText, "Modified File",
                MB_YESNOCANCEL | MB_ICONQUESTION);
            break;

        case edReSynch:         // Passes char * to the name of the file
            sprintf(msgText,
                "The copy of %s on disk is different from the one in memory."
                "  Reload from disk?", (char *)va_arg(arg, char *));

            result = MessageBox(hWnd, msgText, "Resynch",
                MB_YESNO | MB_ICONQUESTION);
            break;

        case edLoadAutoSave:    // Passes char * to the name of the file
            sprintf(msgText,
              "An auto save file exists for %s.  Reload text from it instead?",
              (char *)va_arg(arg, char *));

            result = MessageBox(hWnd, msgText, "Auto Save File Found",
                MB_YESNO | MB_ICONQUESTION);
            break;
    }
    va_end(arg);

    return result;
}
